@@ -290,7 +290,7 @@ void nlg_amcmc::is_correction_bsf(nlg_ssm model, const unsigned int nsim_states,
   double sum_w = 0.0;
   
 #ifdef _OPENMP
-#pragma omp parallel num_threads(n_threads) default(none) shared(Valpha, sum_w) firstprivate(model) 
+#pragma omp parallel num_threads(n_threads) default(shared) firstprivate(model) 
 {
   
   model.engine = sitmo::prng_engine(omp_get_thread_num() + 1);
@@ -392,7 +392,7 @@ void nlg_amcmc::is_correction_psi(nlg_ssm model, const unsigned int nsim_states,
   double sum_w = 0.0;
   
 #ifdef _OPENMP
-#pragma omp parallel num_threads(n_threads) default(none) shared(Valpha, sum_w) firstprivate(model)
+#pragma omp parallel num_threads(n_threads) default(shared) firstprivate(model)
 {
   model.engine = sitmo::prng_engine(omp_get_thread_num() + 1);
   
@@ -562,7 +562,7 @@ posterior_storage = prior_storage + approx_loglik_storage - scales_storage +
 void nlg_amcmc::state_ekf_sample(nlg_ssm model, const unsigned int n_threads, const unsigned int iekf_iter) {
   
 #ifdef _OPENMP
-#pragma omp parallel num_threads(n_threads) default(none) firstprivate(model)
+#pragma omp parallel num_threads(n_threads) default(shared) firstprivate(model)
 {
   model.engine = sitmo::prng_engine(omp_get_thread_num() + 1);
   
